@@ -22,11 +22,43 @@ from matplotlib.lines import Line2D
 from matplotlib.patches import Patch
 from matplotlib.legend_handler import HandlerLine2D
 import pathlib
+import pandas as pd
 
 from cartopy import util as cutil
 from . import utils
 
 class PAGES2k:
+    ''' A bunch of PAGES2k style settings
+    '''
+    archive_types = [
+        'bivalve',
+        'borehole',
+        'coral',
+        'documents',
+        'ice',
+        'hybrid',
+        'lake',
+        'marine',
+        'sclerosponge',
+        'speleothem',
+        'tree',
+    ]
+    markers = ['p', 'p', 'o', 'v', 'd', '*', 's', 's', '8', 'D', '^']
+    markers_dict = dict(zip(archive_types, markers))
+    colors = [np.array([ 1.        ,  0.83984375,  0.        ]),
+              np.array([ 0.73828125,  0.71484375,  0.41796875]),
+              np.array([ 1.        ,  0.546875  ,  0.        ]),
+              np.array([ 0.41015625,  0.41015625,  0.41015625]),
+              np.array([ 0.52734375,  0.8046875 ,  0.97916667]),
+              np.array([ 0.        ,  0.74609375,  1.        ]),
+              np.array([ 0.25390625,  0.41015625,  0.87890625]),
+              np.array([ 0.54296875,  0.26953125,  0.07421875]),
+              np.array([ 1         ,           0,           0]),
+              np.array([ 1.        ,  0.078125  ,  0.57421875]),
+              np.array([ 0.1953125 ,  0.80078125,  0.1953125 ])]
+    colors_dict = dict(zip(archive_types, colors))
+
+class STYLE:
     colors_dict = {
         'coral.calc': sns.xkcd_rgb['yellow'],
         'coral.SrCa': sns.xkcd_rgb['orange'],
@@ -35,12 +67,19 @@ class PAGES2k:
         'ice.melt': sns.xkcd_rgb['pale blue'],
         'ice.d18O': sns.xkcd_rgb['light blue'],
         'ice.dD': sns.xkcd_rgb['sky blue'],
+        'ice.d-excess': sns.xkcd_rgb['sky blue'],
+        'ice.isotope_diffusion': sns.xkcd_rgb['sky blue'],
+        'ice.hybrid': sns.xkcd_rgb['sky blue'],
         'tree.TRW': sns.xkcd_rgb['green'],
         'tree.MXD': sns.xkcd_rgb['forest green'],
         'tree.ENSO': sns.xkcd_rgb['sea green'],
+        'pollen.temp': sns.xkcd_rgb['red'],
+        'land.temp': sns.xkcd_rgb['red'],
+        'ocean.temp': sns.xkcd_rgb['blue'],
         'tas': sns.xkcd_rgb['pale red'],
         'pr': sns.xkcd_rgb['aqua'],
         'speleothem.d18O': sns.xkcd_rgb['light brown'],
+        'speleothem.dD': sns.xkcd_rgb['brown'],
         'bivalve.d18O': sns.xkcd_rgb['gold'],
         'marine.TEX86': sns.xkcd_rgb['brown'],
         'marine.MgCa': sns.xkcd_rgb['brown'],
@@ -49,6 +88,9 @@ class PAGES2k:
         'marine.alkenone': sns.xkcd_rgb['brown'],
         'marine.foram': sns.xkcd_rgb['brown'],
         'marine.diatom': sns.xkcd_rgb['brown'],
+        'marine.dinocyst': sns.xkcd_rgb['brown'],
+        'marine.radiolaria': sns.xkcd_rgb['brown'],
+        'marine.GDGT': sns.xkcd_rgb['brown'],
         'lake.varve_thickness': sns.xkcd_rgb['dark blue'],
         'lake.varve_property': sns.xkcd_rgb['dark blue'],
         'lake.accumulation': sns.xkcd_rgb['dark blue'],
@@ -63,6 +105,7 @@ class PAGES2k:
         'borehole': sns.xkcd_rgb['peach'],
         'hybrid': sns.xkcd_rgb['maroon'],
         'documents': sns.xkcd_rgb['mauve'],
+        'peat.pollen': sns.xkcd_rgb['brown'],
     }
 
     markers_dict = {
@@ -73,12 +116,19 @@ class PAGES2k:
         'ice.melt': '<',
         'ice.d18O': 'd',
         'ice.dD': '>',
+        'ice.d-excess': 'o',
+        'ice.isotope_diffusion': 'h',
+        'ice.hybrid': '8',
         'tree.TRW': '^',
         'tree.MXD': 'v',
         'tree.ENSO': '^',
+        'pollen.temp': 'o',
+        'land.temp': 'o',
+        'ocean.temp': 'v',
         'tas': '^',
         'pr': 'o',
         'speleothem.d18O': 'o',
+        'speleothem.dD': '>',
         'bivalve.d18O': 'o',
         'marine.TEX86': '*',
         'marine.MgCa': 'v',
@@ -87,6 +137,9 @@ class PAGES2k:
         'marine.alkenone': 'h',
         'marine.foram': '8',
         'marine.diatom': '^',
+        'marine.dinocyst': '>',
+        'marine.radiolaria': '<',
+        'marine.GDGT': 's',
         'lake.varve_thickness': 'H',
         'lake.varve_property': 's',
         'lake.accumulation': 'v',
@@ -101,6 +154,7 @@ class PAGES2k:
         'borehole': '8',
         'hybrid': 'P',
         'documents': 'X',
+        'peat.pollen': 'o',
     }
 
 class CartopySettings:
@@ -155,6 +209,8 @@ def setlabel(ax, label, loc=2, borderpad=0.6, **kwargs):
 
 def plot_field_map(field_var, lat, lon, levels=50, add_cyclic_point=True,
                    title=None, title_size=20, title_weight='normal', figsize=[10, 8],
+                   plot_proxydb=False, proxydb=None, proxydb_lgd_kws=None,
+                   proxy_marker=None, proxy_color=None,
                    site_lats=None, site_lons=None, site_marker='o',
                    site_markersize=50, site_color=sns.xkcd_rgb['amber'],
                    projection='Robinson', transform=ccrs.PlateCarree(),
@@ -162,28 +218,63 @@ def plot_field_map(field_var, lat, lon, levels=50, add_cyclic_point=True,
                    lon_ticks=[60, 120, 180, 240, 300], lat_ticks=[-90, -45, 0, 45, 90],
                    land_color=sns.xkcd_rgb['light grey'], ocean_color=sns.xkcd_rgb['light grey'],
                    land_zorder=None, ocean_zorder=None, signif_values=None, signif_range=[0.05, 9999], hatch='..',
-                   clim=None, cmap=None, cmap_under=None, cmap_over=None, extend=None, mode='latlon', add_gridlines=False,
-                   make_cbar=True, cbar_labels=None, cbar_pad=0.05, cbar_orientation='vertical', cbar_aspect=10,
+                   clim=None, cmap=None, cmap_under=None, cmap_over=None, cmap_bad=None, extend=None, mode='latlon', add_gridlines=False,
+                   plot_cbar=True, cbar_labels=None, cbar_pad=0.05, cbar_orientation='vertical', cbar_aspect=10,
                    cbar_fraction=0.15, cbar_shrink=0.5, cbar_title=None, cbar_title_x=0.5, cbar_title_y=1.05,
-                   font_scale=1.5, plot_type=None, fig=None, ax=None):
+                   fig=None, ax=None):
+    ''' Visualize a field on a map.
 
-    if plot_type == 'corr':
-        clim = [-1, 1] if clim is None else clim
-        levels = np.linspace(-1, 1, 21) if levels is None else levels
-        cbar_labels = np.linspace(-1, 1, 11) if cbar_labels is None else cbar_labels
-        cbar_title = 'r' if cbar_title is None else cbar_title
-        extend = 'neither' if extend is None else extend
-        cmap = 'RdBu_r' if cmap is None else cmap
-    elif plot_type == 'R2':
-        clim = [0, 1] if clim is None else clim
-        levels = np.linspace(0, 1, 21) if levels is None else levels
-        cbar_labels = np.linspace(0, 1, 11) if cbar_labels is None else cbar_labels
-        cbar_title = r'R$^2$' if cbar_title is None else cbar_title
-        extend = 'neither' if extend is None else extend
-        cmap = 'Reds' if cmap is None else cmap
-    else:
-        extend = 'both' if extend is None else extend
-        cmap = 'RdBu_r' if cmap is None else cmap
+    Args:
+        field_var (numpy.ndarray): field data array
+        lat (numpy.array): list of latitudes
+        lon (numpy.array): list of longitudes
+        levels (int or list, optional): contour levels. Defaults to 50.
+        add_cyclic_point (bool, optional): if True, add cyclic point to the field. Defaults to True.
+        title (str, optional): title string. Defaults to None.
+        title_size (int, optional): font size of the title string. Defaults to 20.
+        title_weight (str, optional): font weight of the title string. Defaults to 'normal'.
+        figsize (list, optional): figure size. Defaults to [10, 8].
+        site_lats (list, optional): list of latitudes for the sites to be plotted on the map. Defaults to None.
+        site_lons (list, optional): list of longitudes for the sites to be plotted on the map. Defaults to None.
+        site_marker (list, optional): marker for the sites to be plotted on the map. Defaults to 'o'.
+        site_markersize (float, optional): marker size for the sites to be plotted on the map. Defaults to 50.
+        site_color (str, optional): color of the markers for the sites to be plotted on the map. Defaults to sns.xkcd_rgb['amber'].
+        projection (str, optional): map projection. Defaults to 'Robinson'.
+        transform (object, optional): map tranform. Defaults to ccrs.PlateCarree().
+        proj_args (dict, optional): map projection arguments. Defaults to None.
+        latlon_range (list, optional): for regional plotting; equals to [lat_min, lat_max, lon_min, lon_max]. Defaults to None.
+        central_longitude (int, optional): central longitude for the map projection. Defaults to 180.
+        lon_ticks (list, optional): _description_. Defaults to [60, 120, 180, 240, 300].
+        lat_ticks (list, optional): _description_. Defaults to [-90, -45, 0, 45, 90].
+        land_color (str, optional): _description_. Defaults to sns.xkcd_rgb['light grey'].
+        ocean_color (str, optional): _description_. Defaults to sns.xkcd_rgb['light grey'].
+        land_zorder (float, optional): _description_. Defaults to None.
+        ocean_zorder (float, optional): _description_. Defaults to None.
+        signif_values (float, optional): _description_. Defaults to None.
+        signif_range (list, optional): _description_. Defaults to [0.05, 9999].
+        hatch (str, optional): _description_. Defaults to '..'.
+        clim (list, optional): _description_. Defaults to None.
+        cmap (str, optional): colormap. Defaults to None.
+        cmap_under (str, optional): color under the limit. Defaults to None.
+        cmap_over (str, optional): color over the limit. Defaults to None.
+        cmap_bad (str, optional): color for bad values. Defaults to None.
+        extend (str, optional): if True, extend the colorbar. Defaults to None.
+        mode (str, optional): _description_. Defaults to 'latlon'.
+        add_gridlines (bool, optional): _description_. Defaults to False.
+        plot_cbar (bool, optional): if True, plot the colorbar. Defaults to True.
+        cbar_labels (_type_, optional): colorbar labels. Defaults to None.
+        cbar_pad (float, optional): colorbar padding space. Defaults to 0.05.
+        cbar_orientation (str, optional): colorbar orientation. Defaults to 'vertical'.
+        cbar_aspect (int, optional): colorbar aspect. Defaults to 10.
+        cbar_fraction (float, optional): colorbar fraction. Defaults to 0.15.
+        cbar_shrink (float, optional): colorbar shrink. Defaults to 0.5.
+        cbar_title (_type_, optional): colorbar title string. Defaults to None.
+        cbar_title_x (float, optional): colorbar title location x. Defaults to 0.5.
+        cbar_title_y (float, optional): colorbar title location y. Defaults to 1.05.
+        fig (object, optional): `matplotlib.figure`. Defaults to None.
+        ax (object, optional): `matplotlib.axes`. Defaults to None.
+
+    '''
 
     if add_cyclic_point:
         if mode == 'latlon':
@@ -235,8 +326,8 @@ def plot_field_map(field_var, lat, lon, levels=50, add_cyclic_point=True,
         plt.title(title, fontsize=title_size, fontweight=title_weight)
 
     if latlon_range:
-        lon_min, lon_max, lat_min, lat_max = latlon_range
-        ax.set_extent(latlon_range, crs=transform)
+        lat_min, lat_max, lon_min, lon_max = latlon_range
+        ax.set_extent([lon_min, lon_max, lat_min, lat_max], crs=transform)
         lon_formatter = LongitudeFormatter(zero_direction_label=False)
         lat_formatter = LatitudeFormatter()
         ax.xaxis.set_major_formatter(lon_formatter)
@@ -252,7 +343,7 @@ def plot_field_map(field_var, lat, lon, levels=50, add_cyclic_point=True,
 
     ax.add_feature(cfeature.LAND, facecolor=land_color, edgecolor=land_color, zorder=land_zorder)
     ax.add_feature(cfeature.OCEAN, facecolor=ocean_color, edgecolor=ocean_color, zorder=ocean_zorder)
-    ax.coastlines()
+    ax.coastlines(zorder=99)
 
     if add_gridlines:
         ax.gridlines(edgecolor='gray', linestyle=':', crs=transform)
@@ -262,6 +353,8 @@ def plot_field_map(field_var, lat, lon, levels=50, add_cyclic_point=True,
         cmap.set_under(cmap_under)
     if cmap_over is not None:
         cmap.set_over(cmap_over)
+    if cmap_bad is not None:
+        cmap.set_bad(cmap_bad)
 
     if mode == 'latlon':
         im = ax.contourf(lon_c, lat_c, field_var_c, levels, transform=transform, cmap=cmap, extend=extend)
@@ -279,7 +372,7 @@ def plot_field_map(field_var, lat, lon, levels=50, add_cyclic_point=True,
     if signif_values is not None:
         ax.contourf(lon_c, lat_c, signif_values_c, signif_range, transform=transform, hatches=[hatch], colors='none')
 
-    if make_cbar:
+    if plot_cbar:
         cbar = fig.colorbar(im, ax=ax, orientation=cbar_orientation, pad=cbar_pad, aspect=cbar_aspect, extend=extend,
                         fraction=cbar_fraction, shrink=cbar_shrink)
 
@@ -292,11 +385,53 @@ def plot_field_map(field_var, lat, lon, levels=50, add_cyclic_point=True,
     if site_lats is not None and site_lons is not None:
         if type(site_lats) is not dict:
             ax.scatter(site_lons, site_lats, s=site_markersize, c=site_color, marker=site_marker, edgecolors='k',
-                       zorder=99, transform=transform)
+                       zorder=99, transform=transform, cmap=cmap)
         else:
             for name in site_lats.keys():
-                ax.scatter(site_lons[name], site_lats[name], s=site_markersize[name], c=site_color[name], marker=site_marker[name], edgecolors='k',
-                           zorder=99, transform=transform)
+                ax.scatter(site_lons[name], site_lats[name], s=site_markersize[name], c=site_color[name],
+                           marker=site_marker[name], edgecolors='k', zorder=99, transform=transform, cmap=cmap)
+
+    if plot_proxydb:
+        proxydb_lgd_kws = {} if proxydb_lgd_kws is None else proxydb_lgd_kws
+        ptypes = []
+        ptype_labels = []
+        for k, v in proxydb.type_dict.items():
+            ptypes.append(k)
+            ptype_labels.append(f'{k} (n={v})')
+
+        site_lats, site_lons = {}, {}
+        for ptype in ptypes:
+            site_lats[ptype] = []
+            site_lons[ptype] = []
+
+        for _, pobj in proxydb.records.items():
+            site_lats[pobj.ptype].append(pobj.lat)
+            site_lons[pobj.ptype].append(pobj.lon)
+
+        s_plots = []
+        for ptype in ptypes:
+            if ptype in STYLE.markers_dict and proxy_marker is None:
+                marker = STYLE.markers_dict[ptype]
+            else:
+                marker = proxy_marker
+
+            if ptype in STYLE.colors_dict and proxy_color is None:
+                color = STYLE.colors_dict[ptype]
+            else:
+                color = proxy_color
+
+            s_plots.append(
+                ax.scatter(
+                    site_lons[ptype], site_lats[ptype], marker=marker,
+                    c=color, edgecolor='k', s=site_markersize, transform=ccrs.PlateCarree()
+                )
+            )
+
+        ax.legend(
+            s_plots, ptype_labels,
+            **proxydb_lgd_kws,
+        )
+        
 
     return fig, ax
 
@@ -306,8 +441,16 @@ def plot_proxies(df, year=np.arange(2001), lon_col='lon', lat_col='lat', type_co
                  figsize=[10, 10], projection='Robinson', proj_args=None, central_longitude=180, markersize=50,
                  plot_count=False, nrow=2, ncol=1, wspace=0.5, hspace=0.3,
                  lgd_ncol=None, lgd_anchor_upper=(1, 0), lgd_anchor_lower=(1, -0.05),lgd_frameon=False,
-                 enumerate_ax=False, enumerate_prop={'weight': 'bold', 'size': 30}, p=PAGES2k, stock_img=True,
+                 enumerate_ax=False, enumerate_prop={'weight': 'bold', 'size': 30}, p=STYLE, stock_img=True,
                  enumerate_anchor_map=[0, 1], enumerate_anchor_count=[0, 1], map_grid_idx=0, count_grid_idx=-1):
+    ''' Visualize proxies.
+
+    Args:
+        df (pandas.DataFrame): proxy database in `pandas.DataFrame`.
+        year (list): the years for counting proxy numbers.
+        p (object): the style object that contains dictionaries of markers and colors
+
+    '''
 
     fig = plt.figure(figsize=figsize)
 
@@ -371,23 +514,24 @@ def plot_proxies(df, year=np.arange(2001), lon_col='lon', lat_col='lat', type_co
 
     if plot_count:
         ax['count'] = plt.subplot(gs[count_grid_idx])
-        proxy_count = {}
+        df_count = {}
+        for ptype in type_set[::-1]:
+            df_count[ptype] = pd.DataFrame(index=year)
+
         for index, row in df.iterrows():
             ptype = row[type_col]
-            time = row[time_col]
-            time = np.array([int(t) for t in time])
+            time = np.array(row[time_col]).astype(int)
             time = time[~np.isnan(time)]
+            time = time[time<np.max(year)+1]
             time = np.sort(list(set(time)))  # remove the duplicates for monthly data
+            ts = pd.Series(index=time, data=1, name=row['pid'])
+            df_count[ptype] = pd.concat([df_count[ptype], ts], axis=1)
 
-            if ptype not in proxy_count.keys():
-                proxy_count[ptype] = np.zeros(np.size(year))
+        proxy_count = {}
+        for ptype in df_count.keys():
+            proxy_count[ptype] = df_count[ptype].sum(axis=1)
 
-            for k in time:
-                # if k < np.max(year):
-                if k in year:
-                    proxy_count[ptype][k] += 1
-
-        cumu_count = np.zeros(np.size(year))
+        cumu_count = np.zeros_like(year)
         cumu_last = np.copy(cumu_count)
         for ptype in type_set:
             cumu_count += proxy_count[ptype]
@@ -416,7 +560,7 @@ def plot_proxies(df, year=np.arange(2001), lon_col='lon', lat_col='lat', type_co
 def plot_proxy_age_map(df, lon_col='lon', lat_col='lat', type_col='type', time_col='time',
                        title=None, title_weight='normal', font_scale=1.5,
                        figsize=[12, 10], projection=ccrs.Robinson, central_longitude=0, markersize=150,
-                       plot_cbar=True, marker_color=None, transform=ccrs.PlateCarree(), p=PAGES2k,
+                       plot_cbar=True, marker_color=None, transform=ccrs.PlateCarree(), p=STYLE,
                        add_nino34_box=False, add_nino12_box=False, add_box=False, add_box_lf=None, add_box_ur=None):
 
     fig = plt.figure(figsize=figsize)
@@ -1081,9 +1225,12 @@ def plot_volc_cdf(year_volc, anom_volc, anom_nonvolc, anom_nonvolc_draws, value_
         else:
             return ax
 
-def make_lb(name, unit):
+def make_lb(name, unit, wrap=False):
     if unit is not None:
-        lb = f'{name} [{unit}]'
+        if wrap:
+            lb = f'{name}\n[{unit}]'
+        else:
+            lb = f'{name} [{unit}]'
     else:
         lb = f'{name}'
     return lb
@@ -1201,4 +1348,40 @@ def plot_ml_predict(res_dict, figsize=[8, 4], xlabel='Time', ylabel='Value', xli
     if 'fig' in locals():
         return fig, ax
     else:
-        return ax
\ No newline at end of file
+        return ax
+
+def plot_eof(eof, pc, lat, lon, time, eof_title='EOF', pc_title='PC'):
+    fig = plt.figure(figsize=[12, 10])
+    gs = gridspec.GridSpec(2, 1)
+    gs.update(wspace=0.1, hspace=0.2)
+
+    ax = {}
+    ax['mode'] = fig.add_subplot(gs[0, 0], projection=ccrs.Robinson(central_longitude=180))
+    ax['mode'].set_global()
+    ax['mode'].add_feature(cfeature.LAND, color='lightgrey')
+    ax['mode'].add_feature(cfeature.OCEAN, color='white')
+    ax['mode'].coastlines()
+
+    CS = ax['mode'].contourf(
+        lon, lat, eof,
+        levels=np.arange(-1, 1.1, 0.1),
+        vmin=-1, vmax=1,
+        cmap='RdBu_r',
+        transform=ccrs.PlateCarree(),
+        extend='neither',
+    )
+    ax['mode'].set_title(eof_title)
+    cbar = fig.colorbar(CS, shrink=1.)
+    # cbar.ax.set_title(r'[$^{\circ}$C]', y=1.05)
+    cbar.ax.set_yticks(np.arange(-1, 1.1, 0.2))
+
+    ax['pc'] = fig.add_subplot(gs[1, 0])
+    ax['pc'].plot(time, pc)
+    ax['pc'].set_title(pc_title)
+    ax['pc'].set_xlabel('Year')
+    ax['pc'].set_ylabel(pc_title)
+    ax['pc'].grid(ls='--')
+    ax['pc'].spines.top.set_visible(False)
+    ax['pc'].spines.right.set_visible(False)
+
+    return fig, ax
\ No newline at end of file
